/*  
 *  - - - - - - - - - - 
 *   g a l _ b p n 0 0  
 *  - - - - - - - - - - 
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  CEO-based bias-precession-nutation matrix.  
 *  
 *  This routine is an independent translation of the FORTRAN routine  
 *  BPN2000 by Patrick Wallace Annexe to IERS Conventions 2000, Cahpter 5  
 *  
 *  Status:
 *  
 *     canonical model.  
 *  
 *  Given:
 *     x,y                 d        CIP coordinates
 *     s                   d        the quantity s (radians)
 *
 *  Returned:
 *     rbpn             d[3][3]     intermediate-to-celestial matrix ("Q")
 *
 *  Called:
 *
 *     gal_ir                       Initialize an r-matrix to the identity matrix
 *     gal_rz                       Rotate an r-matrix about the z-axis
 *     gal_rxr                      Multiply two r-matrices
 *
 *  This revision:
 *  
 *     2002 November 26 ( c version 2008 April 27 )  
 *  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  

#include <math.h>  
#include "gal_bpn00.h"
#include "gal_ir.h"
#include "gal_rz.h"
#include "gal_rxr.h"
  
void  
gal_bpn00  
 (  
    double x,  
    double y,  
    double s,
    double rbpn[3][3]  
 ) 
 
{

  double x2, y2, r2, r, z, a, axy, rr[3][3], rl[3][3] ;

/*
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
/*
 * Prepare to evaluate expression (10).
 */

  x2 = x * x ;
  y2 = y * y ;
  r2 = x2 + y2 ;
  r = sqrt ( r2 ) ;
  z = sqrt ( 1.0 - r2 ) ;
  a = 1.0 / ( 1.0 + z ) ;
  axy = a * x * y ;

/*
 *  Right-hand matrix.
 */
  
  gal_ir ( rr ) ;
  gal_rz ( s, rr ) ;

/*
 * Left-hand matrix.
 */
  
  rl[0][0] = 1.0 - a * x2 ;
  rl[0][1] = -axy ;
  rl[0][2] = -x ;
  rl[1][0] = -axy ;
  rl[1][1] = 1.0 - a * y2 ;
  rl[1][2] = -y ;
  rl[2][0] = x ;
  rl[2][1] = y ;
  rl[2][2] = 1.0 - a * r2 ;

/*
 * The result is the product of the two matrices.
 */
  
  gal_rxr ( rl, rr, rbpn ) ;

/*  
 * Finished.  
 */  
  
}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
